@@ -17,14 +17,14 @@ package ghidra.app.plugin.core.codebrowser;
 
 import javax.swing.Icon;
 
-import org.apache.commons.lang3.StringUtils;
-
 import docking.ComponentProvider;
 import docking.action.builder.ActionBuilder;
 import docking.tool.ToolConstants;
 import generic.theme.GIcon;
 import ghidra.app.CorePluginPackage;
 import ghidra.app.context.ListingActionContext;
+import ghidra.app.context.NavigatableActionContext;
+import ghidra.app.nav.Navigatable;
 import ghidra.app.plugin.PluginCategoryNames;
 import ghidra.app.plugin.core.codebrowser.SelectEndpointsAction.RangeEndpoint;
 import ghidra.app.plugin.core.table.TableComponentProvider;
@@ -95,11 +95,13 @@ public class CodeBrowserSelectionPlugin extends Plugin {
 				.menuPath(ToolConstants.MENU_SELECTION, "&Clear Selection")
 				.menuGroup(SELECT_GROUP, "b")
 				.helpLocation(new HelpLocation(HelpTopics.SELECTION, "Clear Selection"))
-				.withContext(ListingActionContext.class, true)
+				.withContext(NavigatableActionContext.class, true)
 				.inWindow(ActionBuilder.When.CONTEXT_MATCHES)
-				.enabledWhen(c -> hasSelection(c))
-				.onAction(c -> ((CodeViewerProvider) c.getComponentProvider())
-						.setSelection(new ProgramSelection()))
+				.enabledWhen(NavigatableActionContext::hasSelection)
+				.onAction(c -> {
+					Navigatable n = c.getNavigatable();
+					n.setSelection(new ProgramSelection());
+				})
 				.buildAndInstall(tool);
 
 		new ActionBuilder("Select Complement", getName())
@@ -203,20 +205,6 @@ public class CodeBrowserSelectionPlugin extends Plugin {
 		return provider instanceof CodeViewerProvider;
 	}
 
-	private boolean hasSelection(ListingActionContext c) {
-		if (!hasCodeViewer(c)) {
-			return false;
-		}
-
-		if (c.hasSelection()) {
-			return true;
-		}
-
-		CodeViewerProvider provider = (CodeViewerProvider) c.getComponentProvider();
-		String textSelection = provider.getTextSelection();
-		return !StringUtils.isBlank(textSelection);
-	}
-
 	private GhidraProgramTableModel<Address> createTableModel(Program program,
 			CodeUnitIterator iterator, ProgramSelection selection) {
 
