@@ -47,12 +47,14 @@ pcodeTestDefaults.has_vector = 0
 pcodeTestDefaults.small_build = 0
 pcodeTestDefaults.ld_library_path = ''
 pcodeTestDefaults.toolchain_type = 'gcc'
-pcodeTestDefaults.compile_exe = 'bin/gcc'
-pcodeTestDefaults.objdump_exe = 'bin/objdump'
+pcodeTestDefaults.compile_exe = 'gcc'
+pcodeTestDefaults.objdump_exe = 'objdump'
 pcodeTestDefaults.objdump_option = ''
-pcodeTestDefaults.readelf_exe = 'bin/readelf'
-pcodeTestDefaults.nm_exe = 'bin/nm'
-pcodeTestDefaults.strip_exe = 'bin/strip'
+pcodeTestDefaults.readelf_exe = 'readelf'
+pcodeTestDefaults.exec_dir = 'bin/'
+pcodeTestDefaults.exec_prefix = ''
+pcodeTestDefaults.nm_exe = 'nm'
+pcodeTestDefaults.strip_exe = 'strip'
 pcodeTestDefaults.variants = {'O0': '-O0', 'O3': '-O3'}
 pcodeTestDefaults.proc_test = ''
 pcodeTestDefaults.force = False
