@@ -1007,6 +1007,26 @@ class StructureDB extends CompositeDB implements StructureInternal {
 		return index;
 	}
 
+	/**
+	 * Find insertion index such that the index falls after all zero-length components at the
+	 * specified offset and before any bitfields at that offset.
+	 *
+	 * @param index any defined component index which contains offset.
+	 * @param offset offset within structure.
+	 * @return index of first non-zero-length defined checking in the forward direction.
+	 */
+	private int afterNonZeroComponentsAtOffset(int index, int offset) {
+		int maxIndex = components.size();
+		while (index < maxIndex) {
+			DataTypeComponentDB dtc = components.get(index);
+			if (dtc.getOffset() != offset || dtc.getLength() != 0) {
+				break;
+			}
+			++index;
+		}
+		return index;
+	}
+
 	/**
 	 * Identify defined-component index of the first non-zero-length component which contains the
 	 * specified offset. If only zero-length components exist, the last zero-length component which
@@ -1322,14 +1342,20 @@ class StructureDB extends CompositeDB implements StructureInternal {
 				structLength = offset;
 			}
 
+			// Any component insert at an offset should be placed after any zero-length components 
+			// at the same offset but before any non-zero-length component.
+
 			int index = Collections.binarySearch(components, Integer.valueOf(offset),
 				OffsetComparator.INSTANCE);
 
 			int additionalShift = 0;
 			if (index >= 0) {
 				index = backupToFirstComponentContainingOffset(index, offset);
-				DataTypeComponentDB dtc = components.get(index);
-				additionalShift = offset - dtc.getOffset();
+				index = afterNonZeroComponentsAtOffset(index, offset);
+				if (index < components.size()) {
+					DataTypeComponentDB dtc = components.get(index);
+					additionalShift = offset - dtc.getOffset();
+				}
 			}
 			else {
 				index = -index - 1;
